/*******************************************************************************
 *
 * File Name: LDreferenceClass.h
 * Author: Richard Baxter
 * Project: Generic Ldraw Construct Functions
 * Project Version: 1l3j 2-Sept-08
 *
 *******************************************************************************/



/*LDreferenceClass.cpp: Defines a class for references to submodels/parts/units within a scene file*/

#ifndef HEADER_LD_REFERENCE_CLASS
#define HEADER_LD_REFERENCE_CLASS


#include "LDglobalDefs.h"

#ifdef USE_LRRC
	#include "LRRCmodelClass.h"
#else
	#include <string>
	#include <iostream>
	#include <fstream>
	#include <time.h>
	#include <math.h>
	using namespace std;
#endif



#define DAT_FILE_COLOUR_BLACK (0)
#define DAT_FILE_COLOUR_BLUE (1)
#define DAT_FILE_COLOUR_GREEN (2)
#define DAT_FILE_COLOUR_CYAN (3)
#define DAT_FILE_COLOUR_RED (4)
#define DAT_FILE_COLOUR_MAGENTA (5)
#define DAT_FILE_COLOUR_BROWN (6)
#define DAT_FILE_COLOUR_LIGHTGREY (7)
#define DAT_FILE_COLOUR_DARKGREY (8)
#define DAT_FILE_COLOUR_YELLOW (14)
#define DAT_FILE_COLOUR_WHITE (15)
#define DAT_FILE_COLOUR_ORANGE (25)
#define DAT_FILE_COLOUR_PURPLE (22)
#define DAT_FILE_COLOUR_TRANPARENT_MOD (32)
#define DAT_FILE_DEFAULT_COLOUR (16)
#define DAT_FILE_DEFAULT_COLOUR_EDGELINE (24)


#define DAT_FILE_REF_SUBMODEL_NAME_LENGTH_MAX (128)
#define DAT_FILE_REF_NUM_OF_MATRIX_VALUES (9)
#define DAT_FILE_REF_NUM_OF_COLOUR_VALUES (1)
#define DAT_FILE_REF_NUM_OF_TYPE_VALUES (1)
#define DAT_FILE_REF_NUM_OF_POSITION_VALUES (3)
#define DAT_FILE_REF_NUM_OF_SUBMODEL_VALUES (1)
#define DAT_FILE_REF_NUM_DATA_VALUES (DAT_FILE_REF_NUM_OF_TYPE_VALUES + DAT_FILE_REF_NUM_OF_COLOUR_VALUES + DAT_FILE_REF_NUM_OF_POSITION_VALUES + DAT_FILE_REF_NUM_OF_MATRIX_VALUES + DAT_FILE_REF_NUM_OF_SUBMODEL_VALUES)
#define DAT_FILE_DATA_VALUE_MAX_LENGTH (15) 					//assumes matrix / position data - fraction values have a max size of 10 characters
#define DAT_FILE_DATA_VALUE_RECOMMENDED_LENGTH (6) 				//[negative sign] + [1 value before DP] + [DP] + [3 values after DP]
#define DAT_FILE_REF_MAX_SIZE (DAT_FILE_REF_NUM_OF_TYPE_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH + DAT_FILE_REF_NUM_OF_COLOUR_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH + DAT_FILE_DATA_VALUE_MAX_LENGTH * DAT_FILE_REF_NUM_OF_POSITION_VALUES + DAT_FILE_DATA_VALUE_MAX_LENGTH * DAT_FILE_REF_NUM_OF_MATRIX_VALUES + DAT_FILE_REF_SUBMODEL_NAME_LENGTH_MAX + DAT_FILE_REF_NUM_DATA_VALUES)
					/*formatted;
					#define DAT_FILE_REF_MAX_SIZE
						(DAT_FILE_REF_NUM_OF_TYPE_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH +
						DAT_FILE_REF_NUM_OF_COLOUR_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH +
						DAT_FILE_DATA_VALUE_MAX_LENGTH * DAT_FILE_REF_NUM_OF_POSITION_VALUES +
						DAT_FILE_DATA_VALUE_MAX_LENGTH * DAT_FILE_REF_NUM_OF_MATRIX_VALUES +
						DAT_FILE_REF_SUBMODEL_NAME_LENGTH_MAX +
						DAT_FILE_REF_NUM_DATA_VALUES)
					*/
									/*NB there is 2 (integer) + 3 (fractional) + 9 (fractional) + 1 + 15 (spaces) = a total of 15 values are present*/
#define DAT_FILE_REF_MATRIX_MAX_LENGTH  (DAT_FILE_DATA_VALUE_MAX_LENGTH*DAT_FILE_REF_NUM_OF_MATRIX_VALUES + DAT_FILE_REF_NUM_OF_MATRIX_VALUES)
#define DAT_FILE_REF_POS_COORD_MAX_LENGTH (DAT_FILE_DATA_VALUE_MAX_LENGTH*DAT_FILE_REF_NUM_OF_POSITION_VALUES + DAT_FILE_REF_NUM_OF_POSITION_VALUES)
#ifdef USE_LRRC
	#define DAT_FILE_MAX_NUM_OF_REFERENCES (10000U)	//10 thousand references
#elif defined USE_ANN
	#define DAT_FILE_MAX_NUM_OF_REFERENCES (1000000U)	//1 million references
#else
	#define DAT_FILE_MAX_NUM_OF_REFERENCES (100000U)	//100 thousand references
#endif

	/*new parameters added to parseFile() 18-mar-07*/
#define REFERENCE_TYPE_COMMENT (0)
#define REFERENCE_TYPE_SUBMODEL (1)
#define REFERENCE_TYPE_LINE (2)
#define REFERENCE_TYPE_TRI (3)
#define REFERENCE_TYPE_QUAD (4)
#define REFERENCE_TYPE_OPTIONALLINE (5)
#define REFERENCE_TYPE_UNDEFINED (9)
#define REFERENCE_TYPE_COMMENT (10)
#define PRIM_REF_MAX_NUM_OF_POSITION_VALUES (12)
#define PRIMATIVE_REF_MAX_SIZE (DAT_FILE_REF_NUM_OF_TYPE_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH + DAT_FILE_REF_NUM_OF_COLOUR_VALUES * DAT_FILE_DATA_VALUE_MAX_LENGTH + DAT_FILE_DATA_VALUE_MAX_LENGTH * PRIM_REF_MAX_NUM_OF_POSITION_VALUES)


typedef struct { double x, y, z; } vec;
typedef struct { vec a; vec b; vec c; } mat;
	/*a matrix is defined by 3 vectors side by side*/

class Reference{
private:
	/*There are currently no private attributes of this class*/
public:

	Reference(void); // constructor declaration
	Reference(string referenceName, int referenceColour, bool createNewSubmodel); // constructor declaration
	Reference(bool createNewSubmodel);
	~Reference();	//  and destructor.

	vec relativePosition;
	vec absolutePosition;
	mat deformationMatrix;
	mat absoluteDeformationMatrix;
	int colour;
	string name;
	Reference * next;

		/*new parameters added to parseFile() 18-mar-07*/
	vec vertex1relativePosition;
	vec vertex2relativePosition;
	vec vertex3relativePosition;
	vec vertex4relativePosition;
	vec vertex1absolutePosition;
	vec vertex2absolutePosition;
	vec vertex3absolutePosition;
	vec vertex4absolutePosition;
	int type;

#ifdef USE_LRRC
	/*Additional values used with recursive parser*/
	bool isSubModelReference;
	ModelDetails * subModelDetails;
	Reference * firstReferenceWithinSubModel;
#endif
};

void copyReferencePosition(Reference * r, Reference * referenceToCopy);

int obtainReferencePlayerID(Reference * referenceInThisPhaseStartSceneFile);

#endif

